package milab.lab05.classifierAgent;

import jade.core.Agent;
import jade.core.AID;
import jade.core.behaviours.*;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;

import javax.xml.bind.*;

import java.io.*;
import java.math.BigDecimal;
import java.util.List;


import milab.*;
import milab.lab05.*;
import milab.lab05.language1.DataType;
import milab.lab05.language1.ObjectFactory;
import milab.lab05.libsvm.*;
import milab.lab05.language1.*;
import milab.lab05.classifierAgent.ClassifierAgentGui;

public class ClassifierAgent extends Agent {
	private static final long serialVersionUID = -3289187484920206326L;
	public AID[] webcamAgents;
	svm_model model;
	// The GUI to show Emese
	private ClassifierAgentGui myGui;
	
	public void setup() {
		
		// Create and show the GUI 
	    myGui = new ClassifierAgentGui(this);
	    myGui.show();
	    
		System.out.println("Classifier agent is starting.");
	
		//DF regisztracio
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(getAID());
		ServiceDescription sd = new ServiceDescription();
		sd.setType("classifier");
		sd.setName("classifier");
		dfd.addServices(sd);
		try {
		  DFService.register(this, dfd);
		}
		catch (FIPAException fe) {
		  fe.printStackTrace();
		}

		//A model elksztse
		CreateModel();
		
		addBehaviour(new AskBehaviour(this,3000));
		addBehaviour(new ClassifyBehavior());
	}
	
	protected void takeDown() {
		
		myGui.dispose();
		System.out.println("Classifier agent is terminating.");
		
		try {
			
			DFService.deregister(this);
			
		} catch (FIPAException fe) {
			
			fe.printStackTrace();
		}
		
	}
	
	private void CreateModel()
	{
		//Train the SVM
		svm_train t = new svm_train();
		
		  
		try {
			System.out.println("Training SVM.");

			//Belltjuk a kt osztlyt tartalmaz mitafile-t
			String sample_file_name = "/jade/src/milab/lab05/SampleFiles/minusz3.txt";
			String sample_file_name2 = "/jade/src/milab/lab05/SampleFiles/plussz3.txt";
			
			//Itt belltjuk a tants paramtereit
			double C=1;
			double sigma=1;
			double epsilon=100; // !!!osztlyozs esetn nem szmt!!!
			
			// az kt osztly mintafjljai alapjn el kell kszteni a modell-t
			
			//trainSVM(String file_name, int svm_type, double C, double sigma, double epszilon)
			model = t.trainSVM(sample_file_name,sample_file_name2, svm_parameter.C_SVC, C, sigma, epsilon);
			
			System.out.println("Validating SVM.");
			
			//a bemeneti TESZT file s az eredmnyt tartalmaz kimeneti file
			String validate_file_name = "/jade/src/milab/lab05/SampleFiles/minusz1_plussz1.txt"; //minusz1_plussz1.txt";
			String output_file_name =  "/jade/src/milab/lab05/SampleFiles/output.txt";
						
			BufferedReader input = new BufferedReader(new FileReader(validate_file_name));
			DataOutputStream output = new DataOutputStream(new FileOutputStream(output_file_name));
			
			//A modell validlsa
			svm_predict.predict(input, output, model, 0);
			
			//close the files
			input.close();
			output.close();
			
			System.out.println("SVM model is ready.");
		}
		catch(IOException ex)
		{
			System.out.println("Error reading SVM train data.");
		}		
	}
	
	private class AskBehaviour extends TickerBehaviour {
		private static final long serialVersionUID = 7219759545043260938L;

		public AskBehaviour(Agent agent, long time) {
			super(agent, time);
		}

		protected void onTick() {
			
						
		}
	}
		
	private class ClassifyBehavior extends CyclicBehaviour {
		private static final long serialVersionUID = -4541557963608217717L;

		@SuppressWarnings("unchecked")
		public void action() {
			ACLMessage msg = receive();
			if (msg != null) {
				if(msg.getPerformative()==ACLMessage.REQUEST) {
				    JAXBUnMarshaller jaxbUnMarshaller = new JAXBUnMarshaller();		// Creating an UnMarshaller
				    JAXBElement<DataType> contentElement = (JAXBElement<DataType>)jaxbUnMarshaller.readXMLString(msg.getContent(), "C:/jade/src/milab/lab05/language1/language1.xsd", "milab.lab05.language1");
				    DataType data=contentElement.getValue();
				    
				    int vectorLength = data.getPixel().size();
					 
					svm_node[] x = new svm_node[vectorLength];
		
				    for(int index = 0; index<vectorLength; index ++)
				    {
					    int pixelValue = data.getPixel().get(index);
					    //System.out.println("pixel[" + new Integer(index).toString() + "] =" + new Integer(pixelValue).toString() + "");
					    x[index] = new svm_node();
						x[index].index = index;
						x[index].value = pixelValue;	
				    }
				    
				    double j;
					j = svm.svm_predict(model, x);
					
					myGui.result.setText(Double.toString(j));
					System.out.println("The result is:" + j);

				}
			}
			else {
			  block();
			}
		}
		
	}
}

