package milab.lab05.studentAgent;


import java.io.*;
import java.math.BigDecimal;
import java.util.Vector;

import javax.xml.bind.JAXBElement;

import milab.JAXBMarshaller;
import milab.JAXBUnMarshaller;
import milab.lab05.svm_predict;
import milab.lab05.svm_train;
import milab.lab05.language.*;
import milab.lab05.libsvm.*;
import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.TickerBehaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;

public class StudentAgent extends Agent {
	
	private static final long serialVersionUID = -3289187484920206326L;
	public AID[] teacherAgents;
	//Adatok tmbje
	Vector<Double> vy = new Vector<Double>();
	Vector<svm_node[]> vx = new Vector<svm_node[]>();
	svm_model model;
	//a tanitshoz szksges pontok szma (ennyit vr ssze)
	int trainSampleNum;
	// The GUI to show Emese
	private StudentAgentGui myGui;
	
	public void setup() {
		
	    // Create and show the GUI 
	    myGui = new StudentAgentGui(this);
	    myGui.show();
		
		System.out.println("Student agent is starting.");
		
		Object[] args = getArguments();
		  if (args != null && args.length > 0) {
			  trainSampleNum = Integer.parseInt((String) args[0]);
			  System.out.println("Mintaszm: " + trainSampleNum);
		  }
				
		addBehaviour(new SearchTeachers(this,5000));
		addBehaviour(new AskBehaviour(this,50));
		addBehaviour(new ReceiveBehaviour());
		addBehaviour(new ValidateBehaviour());
		
	}
	
	protected void takeDown() {
		
		    // Deregister from the yellow pages
		    try {
		    	
		      DFService.deregister(this);
		      
		    } catch (FIPAException fe) {
		    	
		      fe.printStackTrace();
		      
		    }

		  	// Close the GUI
		  	myGui.dispose();
		  	
		    // Printout a dismissal message
		    System.out.println("Student-agent " + getAID().getName() + " terminating.");
	}
	
	private class SearchTeachers extends TickerBehaviour {
		public SearchTeachers(Agent agent, long time) {
			super(agent, time);
		}
		private static final long serialVersionUID = 6843089055541089809L;

		protected void onTick() {
			System.out.println("Searching for teachers");
			DFAgentDescription template = new DFAgentDescription();
			ServiceDescription sd = new ServiceDescription();
			sd.setType("teaching");
			template.addServices(sd);
			try {
			  DFAgentDescription[] result = DFService.search(myAgent, template);
			  System.out.println("Found: "+new Integer(result.length).toString()+" teachers.");
			  teacherAgents = new AID[result.length];
			  for (int i = 0; i < result.length; ++i) {
			    teacherAgents[i] = result[i].getName();
			  }
			}
			catch (FIPAException fe) {
			  fe.printStackTrace();
			}
							
		}
	}
	private class AskBehaviour extends TickerBehaviour {
		private static final long serialVersionUID = 7219759545043260938L;

		public AskBehaviour(Agent agent, long time) {
			super(agent, time);
		}

		protected void onTick() {
			
			if (teacherAgents != null)
			{
				//System.out.println("Asking teachers.");
			    ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
				DataType data=new DataType();
				data.setX(new Double(20*Math.random()-10).toString());
				ObjectFactory factory = new ObjectFactory();
				JAXBElement<DataType> contentElement=factory.createData(data);
				JAXBMarshaller marshaller = new JAXBMarshaller();
				msg.setContent(marshaller.generateXMLString("UTF-8", contentElement, "milab.lab05.language"));	        
	
				for(int i=0;i<teacherAgents.length;++i) {
				   	msg.addReceiver(teacherAgents[i]);
				}
				
				if(teacherAgents.length>0) {
				  	myAgent.send(msg);
				  	System.out.println("Request sent.");
				}	
			}
						
		}
	}
	private class ReceiveBehaviour extends CyclicBehaviour {
		private static final long serialVersionUID = 1549324419788895976L;

		public void action() {
			ACLMessage msg = receive();
			if(msg!=null && vy.size() < trainSampleNum) {
				if(msg.getPerformative()==ACLMessage.INFORM) {
					System.out.println("Reply");
					
					JAXBUnMarshaller jaxbUnMarshaller = new JAXBUnMarshaller();		// Creating an UnMarshaller
				    JAXBElement<DataType> contentElement = (JAXBElement<DataType>)jaxbUnMarshaller.readXMLString(msg.getContent(), "./milab/lab05/language/language.xsd", "milab.lab05.language");
				    DataType data=contentElement.getValue();
				    
				    
				    //msg.getSender().getName(); // a sender agent neve
				    
				    double data_x = new Double(data.getX());
				    double data_y = new Double(data.getY());
					
				    //bemenet
				    svm_node[] x = new svm_node[1];
				    x[0] = new svm_node();
				    x[0].index=0;
				    x[0].value = data_x;
				    vx.addElement(x);
				    
				    //kimenet
				    vy.addElement(data_y);
				    
					if(vy.size() == trainSampleNum)
				    {
						System.out.println("Megrkezett a tantshoz szksges " + new Integer(vy.size()).toString() +  "adat.");
						//Start Learning ha elég pont van
				    	addBehaviour(new LearnBehaviour());
				    }
					
				}
			}
			else {
				block();
			}			
		}
	}
	
	private class LearnBehaviour extends OneShotBehaviour {
		private static final long serialVersionUID = -1410798594523499910L;

		public void action() 
			{
			System.out.println("Tanitas.");
			
			svm_problem prob = new svm_problem();
			prob.l = vy.size();
			prob.x = new svm_node[prob.l][];
			for(int i=0;i<prob.l;i++)
				prob.x[i] = vx.elementAt(i);
			prob.y = new double[prob.l];
			for(int i=0;i<prob.l;i++)
				prob.y[i] = vy.elementAt(i);
			
			
			//Train the SVM
			svm_train t = new svm_train();
			
			//-- ha iteratvan kell a C-t hangolni, akkor ide kell egy cikust pteni, aminek a msodik felben
			//-- validlni kell (lsd. ValidateBehavior). Ehhez mintt pldul a bemeneti mintafjlbl lehet kivlogatni
			//-- utna ki kell szmtani a hibt s a legjobb C-t (legkissebb hiba) kell megadni a vgso
			//-- modell kialaktshoz
			
			//Itt belltjuk a tants paramtereit
			double C=100;
			double sigma=10;
			double epsilon=1e-3; //1e-3 !!!osztlyozs esetn nem szmt!!!
			
			//trainSVM(String file_name, int svm_type, double C, double sigma, double epszilon)
			model = t.trainSVM(prob, svm_parameter.EPSILON_SVR, C, sigma, epsilon);
			
			System.out.println("Tants ksz.");
			
			System.out.println("Validating SVM.");
			
			try {
			//a bemeneti TESZT file s az eredmnyt tartalmaz kimeneti file
			String validate_file_name = "/jade/src/milab/lab05/SampleFiles/sinc_test.txt"; 
			String output_file_name =  "/jade/src/milab/lab05/SampleFiles/output.txt";
			// a kimeneti fljt nyissa meg excel-ben s ksztsen egy grafikont (line) 
			// ezen vizulisan is ellenorizheti a megoldst
						
			BufferedReader input = new BufferedReader(new FileReader(validate_file_name));
			DataOutputStream output = new DataOutputStream(new FileOutputStream(output_file_name));
			
			//A modell validlsa
			svm_predict.predict(input, output, model, 0);
			
			//close the files
			input.close();
			output.close();
			}
			catch(IOException ex)
			{
				System.out.println("Error reading SVM validate data.");
			}
			
			addBehaviour(new MSEBehaviour());
			   	
	    	vx.clear();
	    	vy.clear();
			}
	}
	
	private class ValidateBehaviour extends OneShotBehaviour {
		private static final long serialVersionUID = -8074505657074971756L;

		public void action() 
			{
			
			}
	}
	
	private class MSEBehaviour extends OneShotBehaviour {
		private static final long serialVersionUID = -7404246889902284012L;

		public void action() {
			//sszehasonltja a tanult modellt a referencival
			double x;
			double mse=0;
			svm_node node[] = new svm_node [1];
			node[0]=new svm_node();
			node[0].index=0;
			int n=0;
			for (x=-10;x<10;x+=0.05) {
				node[0].value=x;
				mse+=Math.pow(reference(x)-svm.svm_predict(model, node), 2.0);
				n++;
			}
			myGui.emese.setText(Double.toString(mse/n));
			System.out.println("MSE: "+Double.toString(mse/n));
		}
		
		protected double reference(double x) {
			double val;
			try {
				if (x==0)
				{
					val=1;	
				}
				else
				{
					val=Math.sin(x)/x ;
				} 
			}
			catch (ArithmeticException ex) {
				val=1;
			}
			return val;			
		}
	}
}
