package milab.lab05.teacherAgent;

import java.math.BigDecimal;

import javax.xml.bind.JAXBElement;

import milab.JAXBMarshaller;
import milab.JAXBUnMarshaller;
import milab.lab05.language.*;

import jade.core.Agent;
import jade.core.behaviours.*;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;

public class TeacherAgent extends Agent {
	private static final long serialVersionUID = 9160869800704161374L;
	private double noise;
	
	protected void setup() {
		System.out.println("Teacher agent is initializing.");
		
		String strNoise;
		// Get the title of the book to buy as a start-up argument
		Object[] args = getArguments();
		  if (args != null && args.length > 0) {
			  strNoise = (String) args[0];
			  System.out.println("Teacher noise is:" + strNoise);
			  noise = new Double(strNoise);
		  }

		//behaviour hozzáadása
		addBehaviour(new TeachBehavior());
		
		//DF regisztracio
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(getAID());
		ServiceDescription sd = new ServiceDescription();
		sd.setType("teaching");
		sd.setName("teacher");
		dfd.addServices(sd);
		try {
		  DFService.register(this, dfd);
		}
		catch (FIPAException fe) {
		  fe.printStackTrace();
		}
	}
	protected void takeDown() {
		System.out.println("Teacher agent is terminating.");
		try {
			DFService.deregister(this);
		}
		catch (FIPAException fe) {
			fe.printStackTrace();
		}
	}
	
	private double f(double x) {
		double val;
		try {
			if (x==0)
			{
				val=1;	
			}
			else
			{
				double u = Math.random();
				double z = 1/Math.sqrt(2*Math.PI);
				val=Math.sin(x)/x + noise * z * Math.exp(-1* u*u/2);
			}
		}
		catch (ArithmeticException ex) {
			val=1;
		}
		return val;
	}

	private class TeachBehavior extends CyclicBehaviour {
		private static final long serialVersionUID = 5995819418110314586L;
		@SuppressWarnings("unchecked")
		public void action() {
			ACLMessage msg = receive();
			if (msg != null) {
				if(msg.getPerformative()==ACLMessage.REQUEST) {
				    JAXBUnMarshaller jaxbUnMarshaller = new JAXBUnMarshaller();		// Creating an UnMarshaller
				    JAXBElement<DataType> contentElement = (JAXBElement<DataType>)jaxbUnMarshaller.readXMLString(msg.getContent(), "./milab/lab05/language/language.xsd", "milab.lab05.language");
				    DataType data=contentElement.getValue();
				    double x=new Double(data.getX());
				    
				    //ide jön a függvény
				    double y=f(x);
				    
				    ACLMessage reply=msg.createReply();
				    reply.setPerformative(ACLMessage.INFORM);
				    ObjectFactory factory = new ObjectFactory();
				    data.setY(new Double(y).toString());
				    contentElement=factory.createData(data);
				    JAXBMarshaller marshaller = new JAXBMarshaller();
				    reply.setContent(marshaller.generateXMLString("UTF-8", contentElement, "milab.lab05.language"));
				    myAgent.send(reply);
				}
			}
			else {
			  block();
			}
		}
		
	}
}
