package milab.lab05.webcamAgent;


import java.awt.Image;
import java.io.IOException;
import java.util.Iterator;
import java.util.Vector;

import javax.media.Buffer;
import javax.media.CannotRealizeException;
import javax.media.CaptureDeviceInfo;
import javax.media.CaptureDeviceManager;
import javax.media.Format;
import javax.media.Manager;
import javax.media.MediaLocator;
import javax.media.NoPlayerException;
import javax.media.Player;
import javax.media.control.FormatControl;
import javax.media.control.FrameGrabbingControl;
import javax.media.format.VideoFormat;
import javax.media.util.BufferToImage;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

//import com.sun.media.protocol.vfw.VFWCapture;

public class JMFWebcam implements Webcam{
	
	protected CaptureDeviceInfo webCamDeviceInfo;
    protected MediaLocator ml;
    protected Player player;
    protected FormatControl formatControl;
    protected Format[] videoFormats;
    protected JFrame mw;
	protected boolean started;

    
    public JMFWebcam(JFrame mw) throws Exception{
    	this.mw=mw;
    	init();
    }
    
	private boolean init() throws Exception {
	    Vector<CaptureDeviceInfo> cams = autoDetect();
	    
	    if (cams.size()>0)
        {
            if (cams.size()==1)
            {
            	webCamDeviceInfo = cams.get(0);
            }
            else
            {
            	String[] names = new String[cams.size()];
            	Iterator<CaptureDeviceInfo> i=cams.iterator();
            	int j=0;
            	while(i.hasNext())
            	{
            		names[j++]=i.next().getName();
            	}
                Object selected = JOptionPane.showInputDialog(mw,"Select Video format","Capture format selection",JOptionPane.INFORMATION_MESSAGE,null,names,names[0]);
                if (selected!=null)
                {
                	for(j=0;j<names.length;j++)
                	{
                		if (names[j].equals(selected))
                		{
                			webCamDeviceInfo = cams.get(j);
                		}
                	}
                }
            }
        }
        else
        {
        	System.err.println("Error : No devices");
        }
	    return false;
    }
	
	public void initCam() throws Exception{
		if (webCamDeviceInfo!=null)
        {
            try
            {
                ml = webCamDeviceInfo.getLocator();
                if (ml!=null)
                {
                	player = Manager.createRealizedPlayer(ml);
                }
            }catch(CannotRealizeException nre)
            {
            	System.out.println(nre.toString());
            	throw new Exception();
            }catch(NoPlayerException e)
            {
            	System.out.println(e.toString());
            	throw new Exception();
            }catch(IOException e)
            {
                System.out.println("Error connecting to [" + webCamDeviceInfo.getName() + "] : " + e.getMessage());
            	throw new Exception();
            }
        }
	}
	
	public void startCam(){
		startCam(-1);
	}
	
	public void startCam(int limit) {
		if (player!=null && started == false)
        {
            player.start();
            started = true;
            if (limit > 0)
            {
	            formatControl = (FormatControl)player.getControl("javax.media.control.FormatControl");
	            videoFormats = webCamDeviceInfo.getFormats();
	            int minheight = -1;
	            VideoFormat minFormat = null;
	            for (int i = 0; i < videoFormats.length; i++){
		            if (videoFormats[i] instanceof VideoFormat){
		            	VideoFormat vf = (VideoFormat)videoFormats[i];
		            	if (minFormat == null || vf.getSize().height >= limit && vf.getSize().height < minheight){
		            		minFormat = vf;
		            		minheight = vf.getSize().height;
		            	}
		            }
	            }
            
	            if (minFormat != null)
	            	formatControl.setFormat(minFormat);
            } 
            
            /* Format currFormat = formatControl.getFormat();
 
                        visualComponent = player.getVisualComponent();
                        if ( visualComponent != null )
                        {
                            visualContainer.add ( visualComponent, BorderLayout.CENTER );
 
                            if ( currFormat instanceof VideoFormat )
                            {
                                 currentFormat = (VideoFormat)currFormat;
                                 imageSize = currentFormat.getSize();
                                 visualContainer.setPreferredSize ( imageSize );
                                 setSize ( imageSize.width, imageSize.height + statusBar.getHeight() + toolbar.getHeight() );
                            }
                            else
                            {
                                System.err.println("Error : Cannot get current video format");
                            }
 
                            invalidate();
                            pack();
                            return ( true );
                        }
                        else
                        {
                            System.err.println("Error : Could not get visual component");
                            return ( false );
                        }*/
        }
	}

	@SuppressWarnings("unchecked")
	private Vector<CaptureDeviceInfo> autoDetect() {
		Vector<CaptureDeviceInfo> list = CaptureDeviceManager.getDeviceList(null);
		 
        CaptureDeviceInfo cdi = null;
        String name;
        Vector<CaptureDeviceInfo> devices = new Vector<CaptureDeviceInfo>();
 
        if (list!=null)
        {
        	Iterator<CaptureDeviceInfo> i=list.iterator();
        	while(i.hasNext())
        	{
        		cdi = i.next();
        		name = cdi.getName();
        		if (name.startsWith("vfw:"))
        		{
        			devices.add(cdi);
        		}
        	}
        }
        else
        {
            /*for (int i=0;i<10;i++)
            {
                try
                {
                    name = VFWCapture.capGetDriverDescriptionName(i);
                    if (name!=null && name.length()>1)
                    {
                        cdi = com.sun.media.protocol.vfw.VFWSourceStream.autoDetect(i);
                        if (cdi!=null)
                        {
                            devices.add(cdi);
                        }
                    }
                }
                catch (Exception e)
                {}
            }*/
        }
        return (devices);
	}
	
    private Buffer getFrameBuffer()
    {
        if(player!=null)
        {
            FrameGrabbingControl fgc = (FrameGrabbingControl)player.getControl("javax.media.control.FrameGrabbingControl");
            if (fgc!=null)
            {
                return(fgc.grabFrame());
            }
            else
            {
                System.out.println("Error : FrameGrabbingControl is null");
                return(null);
            }
        }
        else
        {
            System.out.println("Error : Player is null");
            return(null);
        }
    }
 
    public Image getImage()
    {
        Buffer buffer = getFrameBuffer();
        if (buffer!=null)
        {
            BufferToImage btoi = new BufferToImage ((VideoFormat)buffer.getFormat());
            if (btoi!=null)
            {
                Image image = btoi.createImage(buffer);
                if (image!=null)
                {
                    return(image);
                }
                else
                {
                    return(null);
                }
            }
            else
            {
                System.out.println ("Error : cannot create BufferToImage instance");
                return(null);
            }
        }
        else
        {
            System.out.println ("Error : Buffer grabbed is null");
            return(null);
        }
    }
    
    public void stopCam(){
    	player.close();
    }
    
    public void closePlayer(){
    	if ( player != null )
        {
            player.close();
            player.deallocate();
            player = null;
        }
    }
}
