﻿package milab.lab05.webcamAgent;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.*;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.gui.GuiAgent;
import jade.gui.GuiEvent;
import jade.lang.acl.ACLMessage;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.math.BigDecimal;

import javax.xml.bind.JAXBElement;

import milab.JAXBMarshaller;
import milab.lab05.language1.DataType;
import milab.lab05.language1.ObjectFactory;



public class WebcamAgent extends GuiAgent implements WindowListener, ActionListener {

	private static final long serialVersionUID = 4339376932233098344L;
	protected Webcam cam;
	protected WebcamAgentFrame frame;
	protected Boolean running;
	protected BufferedImage lowResImage;
	protected boolean[] array;
	static final int INTERVAL = 40;
	public static int W = 320;
	public static int H = 240;
	public static int SMALL_W = 10; //10*10-es kép lesz
	public static int SMALL_H = 10;
	public AID[] classifierAgents;
	
	@Override
	protected void setup()
	{
		System.out.println("Webcam agent is initializing.");

		
		//DF regisztracio
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(getAID());
		ServiceDescription sd = new ServiceDescription();
		sd.setType("webcam");
		sd.setName("webcam");
		dfd.addServices(sd);
		try {
		  DFService.register(this, dfd);
		}
		catch (FIPAException fe) {
		  fe.printStackTrace();
		}
		
		frame = new WebcamAgentFrame(this);
		frame.addWindowListener(this);
		running = Boolean.FALSE;
		try{
			cam = new JMFWebcam(frame);
			cam.initCam();
			cam.startCam();
			frame.setVisible(true);
		}catch(Exception e){
			System.out.println("Can't creat cam! Exiting...");
			doDelete();
			return;
		}
		start();
		
		addBehaviour(new SearchClassifiers(this,3000));
	}
		
	private class SearchClassifiers extends TickerBehaviour {
		public SearchClassifiers(Agent agent, long time) {
			super(agent, time);
		}
		private static final long serialVersionUID = 6843089055541089809L;

		protected void onTick() {
			System.out.println("Searching for classifiers...");
			DFAgentDescription template = new DFAgentDescription();
			ServiceDescription sd = new ServiceDescription();
			sd.setType("classifier");
			template.addServices(sd);
			try {
			  DFAgentDescription[] result = DFService.search(myAgent, template);
			  System.out.println("Found: "+new Integer(result.length).toString()+" classifiers.");
			  classifierAgents = new AID[result.length];
			  for (int i = 0; i < result.length; ++i) {
			    classifierAgents[i] = result[i].getName();
			  }
			}
			catch (FIPAException fe) {
			  fe.printStackTrace();
			}
		}
	}
	 
	public void start(){
		ShotGrabber sg = new ShotGrabber();
		sg.start();
	}
	
	public void end() {
		synchronized (running) {
			running.notify();
			running = Boolean.FALSE;
		}
	}
	
	/**
	 * A kamerát lezárja kilépéskor
	 */
	@Override
	public void takeDown(){
		
		System.out.println("Webcam agent is terminating.");
		
		end();
		
		if (frame != null){
			
			frame.dispose();
		
		}
		
		if (cam != null) {
			
			cam.closePlayer();
			
		}
		
		try {
			
			DFService.deregister(this);
			
		} catch (FIPAException fe) {
			
			fe.printStackTrace();
		}
		
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == "Grab it!"){
			end();
			frame.grabbed();
			synchronized (running) {
				array = new boolean[SMALL_W*SMALL_H];
				for (int i = 0; i< SMALL_H; i++){
					for (int j = 0; j < SMALL_W; j++){
						array[SMALL_H*i + j] = ((lowResImage.getRGB(j, i)) == -1);
					}
				}
			}
			postGuiEvent(new GuiEvent(frame, 1));
		}else{
			start();
			frame.startAgain();
		}
	}

	@Override
	protected void onGuiEvent(GuiEvent ge) {
		if (ge.getType() == 1){
			synchronized (running) {
				//Send image
				addBehaviour(new SendImageBehaviour());	
			}
		}else if(ge.getType() == 0){
			doDelete();
		}
	}
	
	
	private class SendImageBehaviour extends OneShotBehaviour {
		private static final long serialVersionUID = -1410798594523499910L;

		public void action() 
			{
				System.out.println("Sending image.");
			    ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
				DataType data=new DataType();
				
				for(int index = 0;index < array.length;index++)
				{
					if(array[index])
					{
						data.getPixel().add(1);
					}
					else
					{
						data.getPixel().add(0);
					}
				}
				
				ObjectFactory factory = new ObjectFactory();
				JAXBElement<DataType> contentElement=factory.createData(data);
				JAXBMarshaller marshaller = new JAXBMarshaller();
				msg.setContent(marshaller.generateXMLString("UTF-8", contentElement, "milab.lab05.language1"));	        
	
				for(int i=0;i<classifierAgents.length;++i) {
				   	msg.addReceiver(classifierAgents[i]);
				}
				if(classifierAgents.length>0) {
				  	myAgent.send(msg);
				   	System.out.println("Image sent.");
				}
			}
	}
	
	public void windowActivated(WindowEvent e) {	
	}

	public void windowClosed(WindowEvent e) {
	}

	public void windowClosing(WindowEvent e) {
		end();
		if (cam != null)
		{
			cam.stopCam();
			cam.closePlayer();
  	  		cam = null;
  	  		
  	  	}
		if (frame != null)
		{
			frame.dispose();
			frame = null;
		}
		postGuiEvent(new GuiEvent(frame, 0));
	}
	public void windowDeactivated(WindowEvent e) {
	}

	public void windowDeiconified(WindowEvent e) {
	}

	public void windowIconified(WindowEvent e) {
	}

	public void windowOpened(WindowEvent e) {
	}
	
	protected class ShotGrabber extends Thread{
         
		public ShotGrabber() {
			synchronized (running) {
				running = Boolean.TRUE;	
			}
        }
 
		public void run() {
	     	while(running)
	     	{
	     		Image image=cam.getImage();
	     		if (image!=null){
	     			Image smallImage = image.getScaledInstance(W, H, java.awt.Image.SCALE_DEFAULT);
	     			BufferedImage bi = new BufferedImage(WebcamAgent.W,WebcamAgent.H,BufferedImage.TYPE_INT_RGB);
	    			Graphics2D big = bi.createGraphics();
	    			big.drawImage(smallImage, null, null);
	    			
	    			BufferedImage sqBufferedImage = bi.getSubimage((WebcamAgent.W-WebcamAgent.H)/2,0, WebcamAgent.H, WebcamAgent.H);    			
	    			BufferedImage smallSqBufferedImage = sqBufferedImage.getSubimage(WebcamAgent.H/4, WebcamAgent.H/4, WebcamAgent.H/2, WebcamAgent.H/2); 
	    			
	    			BufferedImage bwbi = new BufferedImage(WebcamAgent.H/2,WebcamAgent.H/2,BufferedImage.TYPE_BYTE_BINARY);
	    			big = bwbi.createGraphics();
	    			big.drawImage(smallSqBufferedImage, null, null);
	    			
	    			Image lrImage = bwbi.getScaledInstance(WebcamAgent.SMALL_W, WebcamAgent.SMALL_H, Image.SCALE_AREA_AVERAGING);    			
	    			lowResImage = new BufferedImage(SMALL_W,SMALL_H,BufferedImage.TYPE_BYTE_BINARY);
	    			big = lowResImage.createGraphics();
	    			big.drawImage(lrImage, null, null);
	    			
	    			
	    			frame.setImage(sqBufferedImage);
	    			frame.setSmallImage(smallSqBufferedImage);
	    			BufferedImage smallLowResImage = new BufferedImage(WebcamAgent.H/2,WebcamAgent.H/2,BufferedImage.TYPE_BYTE_BINARY);
	    			big = smallLowResImage.createGraphics();
	    			for (int i = 0; i< SMALL_H; i++){
	    				for (int j = 0; j < SMALL_H; j++){
	    					{
	    						big.setColor(new Color(lowResImage.getRGB(i, j)));
	    						big.fillRect(i*12, j*12, 12, 12);
	    					}
	    				}
	    			}
	    			
	     			frame.setSmallLowResImage(smallLowResImage);
	     		}
	 			synchronized (running) {
			    	try{
			    		running.wait(INTERVAL);
			    	}catch(Exception e){
			    		running = false;
			    	}
	 			}
	     	} 
		}
	}

}
